@@ -65,18 +65,27 @@ class LinuxEfiBootConfig(bootcfg.EfiBootConfig):
         var.set_boot_order(self.blist)
         self.varstore.set_variable(var)
 
+    def linux_loader_info(self):
+        var = self.varstore.get_variable('LoaderInfo', guids.LoaderInfo)
+        if not var:
+            return None
+        return ucs16.from_ucs16(var.data)
+
     def linux_init(self):
         self.varstore = linux.LinuxVarStore()
         self.bootorder = self.linux_read_variable('BootOrder')
         self.bootcurrent = self.linux_read_variable('BootCurrent')
         self.bootnext = self.linux_read_variable('BootNext')
         self.parse_boot_variables()
-        self.add_unused_entries(self.varstore.scan[guids.EfiGlobalVariable])
+        self.add_unused_entries(self.varstore.scan.get(guids.EfiGlobalVariable))
         for nr in self.bentr.keys():
             var = self.linux_read_variable(f'Boot{nr:04X}')
             if var:
                 self.bentr[nr] = bootentry.BootEntry(data = var.data)
 
+        sb = self.linux_read_variable('SecureBoot')
+        if sb and sb.data[0]:
+            self.secureboot = True
 
 class LinuxEfiFile:
     """ class representing a file on a linux file system which we want reference in efi """
@@ -178,9 +187,16 @@ class LinuxOsInfo(OsRelease):
 
     def esp_path(self):
         if not self.esp:
-            result = subprocess.run([ 'bootctl', '--print-esp-path' ],
-                                    stdout = subprocess.PIPE, check = True)
-            self.esp = result.stdout.decode().strip('\n')
+            try:
+                result = subprocess.run([ 'bootctl', '--print-esp-path' ],
+                                        stdout = subprocess.PIPE, check = True)
+            except (FileNotFoundError, subprocess.CalledProcessError):
+                # If bootctl is not installed find the EFI directory at the usual mount points
+                for candidate in '/efi', '/boot/efi', '/boot/EFI', '/boot':
+                    if os.path.isdir(candidate + '/EFI/BOOT'):
+                        self.esp = candidate
+            else:
+                self.esp = result.stdout.decode().strip('\n')
         return self.esp
 
     def shim_path(self):
