@@ -9,6 +9,7 @@ import json
 import logging
 import argparse
 
+from virt.firmware.efi import guids
 from virt.firmware.efi import efivar
 from virt.firmware.efi import efijson
 from virt.firmware.efi import devpath
@@ -17,6 +18,7 @@ from virt.firmware.efi import certs
 
 from virt.firmware.varstore import aws
 from virt.firmware.varstore import jstore
+from virt.firmware.varstore import authfiles
 from virt.firmware.varstore import autodetect
 
 
@@ -81,28 +83,38 @@ def main():
                         help = 'set SbatLevel variable', metavar = 'FILE')
 
     pgroup = parser.add_argument_group('Secure boot setup options')
-    pgroup.add_argument('--set-pk', dest = 'pk',  nargs = 2,
-                        help = 'set PK to x509 cert, loaded in pem format ' +
+    pgroup.add_argument('--set-pk', '--set-pk-cert', dest = 'pk',  nargs = 2,
+                        help = 'set PK to x509 cert, loaded in pem or der format ' +
                         'from FILE and with owner GUID',
                         metavar = ('GUID', 'FILE'))
-    pgroup.add_argument('--add-kek', dest = 'kek',  action = 'append', nargs = 2,
-                        help = 'add x509 cert to KEK, loaded in pem format ' +
+    pgroup.add_argument('--add-kek', '--add-kek-cert', dest = 'kek',  action = 'append', nargs = 2,
+                        help = 'add x509 cert to KEK, loaded in pem or der format ' +
                         'from FILE and with owner GUID, can be specified multiple times',
                         metavar = ('GUID', 'FILE'))
-    pgroup.add_argument('--add-db', dest = 'db',  action = 'append', nargs = 2,
-                        help = 'add x509 cert to db, loaded in pem format ' +
-                        'from FILE and with owner GUID, can be specified multiple times',
-                        metavar = ('GUID', 'FILE'))
-    pgroup.add_argument('--set-dbx', dest = 'dbx',
-                        help = 'initialize dbx with update from FILE', metavar = 'FILE')
-    pgroup.add_argument('--add-mok', dest = 'mok',  action = 'append', nargs = 2,
-                        help = 'add x509 cert to MokList, loaded in pem format ' +
+    pgroup.add_argument('--add-db', '--add-db-cert', dest = 'db',  action = 'append', nargs = 2,
+                        help = 'add x509 cert to db, loaded in pem or der format ' +
                         'from FILE and with owner GUID, can be specified multiple times',
                         metavar = ('GUID', 'FILE'))
     pgroup.add_argument('--add-db-hash', dest = 'db_hash',  action = 'append', nargs = 2,
                         help = 'add sha256 HASH to db, with owner GUID, ' +
                         'can be specified multiple times',
                         metavar = ('GUID', 'HASH'))
+    pgroup.add_argument('--set-dbx', dest = 'dbx',
+                        help = 'initialize dbx with update from FILE', metavar = 'FILE')
+    pgroup.add_argument('--add-dbx', dest = 'add_dbx', action = 'append',
+                        help = 'append dbx update from FILE', metavar = 'FILE')
+    pgroup.add_argument('--add-dbx-cert', dest = 'dbx_cert',  action = 'append', nargs = 2,
+                        help = 'add x509 cert to dbx, loaded in pem or der format ' +
+                        'from FILE and with owner GUID, can be specified multiple times',
+                        metavar = ('GUID', 'FILE'))
+    pgroup.add_argument('--add-dbx-hash', dest = 'dbx_hash',  action = 'append', nargs = 2,
+                        help = 'add sha256 HASH to dbx, with owner GUID, ' +
+                        'can be specified multiple times',
+                        metavar = ('GUID', 'HASH'))
+    pgroup.add_argument('--add-mok', '--add-mok-cert', dest = 'mok', action = 'append', nargs = 2,
+                        help = 'add x509 cert to MokList, loaded in pem or der format ' +
+                        'from FILE and with owner GUID, can be specified multiple times',
+                        metavar = ('GUID', 'FILE'))
     pgroup.add_argument('--add-mok-hash', dest = 'mok_hash',  action = 'append', nargs = 2,
                         help = 'add sha256 HASH to MokList, with owner GUID, ' +
                         'can be specified multiple times',
@@ -116,9 +128,15 @@ def main():
                         help = 'enroll using specified certificate', metavar = "CERT")
     pgroup.add_argument('--enroll-generate', dest = 'enroll_generate',
                         help = 'enroll using generated cert with given common name', metavar = "CN")
+    pgroup.add_argument('--enroll-mgmt', dest = 'enroll_mgmt',
+                        action = 'store_true', default = False,
+                        help = 'enroll using external management guid')
     pgroup.add_argument('--no-microsoft', dest = 'microsoft',
                         action = 'store_false', default = True,
-                        help = 'do not add microsoft keys')
+                        help = 'do not add microsoft keys to db')
+    pgroup.add_argument('--microsoft-kek', dest = 'ms_kek',
+                        choices = [ 'none', '2011', '2023', 'all' ], default = 'all',
+                        help = 'choose microsoft KEK keys to enroll')
     pgroup.add_argument('--distro-keys', dest = 'distro', type = str, action = 'append',
                         help = 'add ca keys for DISTRO', metavar = 'DISTRO')
     pgroup.add_argument('--distro-list', dest = 'distrolist',
@@ -135,6 +153,9 @@ def main():
     pgroup.add_argument('-v', '--verbose', dest = 'verbose',
                         action = 'store_true', default = False,
                         help = 'print varstore verbosely')
+    pgroup.add_argument('--hashes', dest = 'hashes',
+                        action = 'store_true', default = False,
+                        help = 'print signature db sha256 hashes')
     pgroup.add_argument('-x', '--hexdump', dest = 'hexdump',
                         action = 'store_true', default = False,
                         help = 'print variable hexdumps')
@@ -147,6 +168,8 @@ def main():
                         help = 'write aws vars to FILE', metavar = 'FILE')
     pgroup.add_argument('--output-json', dest = 'output_json', type = str,
                         help = 'write json dump to FILE', metavar = 'FILE')
+    pgroup.add_argument('--output-auth', dest = 'output_auth', type = str,
+                        help = 'write *.auth files to DIR', metavar = 'DIR')
     options = parser.parse_args()
 
     logging.basicConfig(format = '%(levelname)s: %(message)s',
@@ -238,17 +261,24 @@ def main():
             varlist[key] = item
 
     if options.redhat:
-        varlist.enroll_platform_redhat()
+        varlist.enroll_platform_redhat(ms_kek = options.ms_kek)
         if options.microsoft:
             varlist.add_microsoft_keys()
 
     if options.enroll_cert:
-        varlist.enroll_platform_with_cert(options.enroll_cert)
+        varlist.enroll_platform_with_cert(options.enroll_cert,
+                                          ms_kek = options.ms_kek)
         if options.microsoft:
             varlist.add_microsoft_keys()
 
     if options.enroll_generate:
-        varlist.enroll_platform_generate(options.enroll_generate)
+        varlist.enroll_platform_generate(options.enroll_generate,
+                                         ms_kek = options.ms_kek)
+        if options.microsoft:
+            varlist.add_microsoft_keys()
+
+    if options.enroll_mgmt:
+        varlist.enroll_managed_platform(ms_kek = options.ms_kek)
         if options.microsoft:
             varlist.add_microsoft_keys()
 
@@ -267,6 +297,10 @@ def main():
         for item in options.db:
             varlist.add_cert('db', item[0], item[1])
 
+    if options.db_hash:
+        for item in options.db_hash:
+            varlist.add_hash('db', item[0], item[1])
+
     if options.dbx:
         logging.info('reading dbx update from %s', options.dbx)
         with open(options.dbx, 'rb') as f:
@@ -274,14 +308,29 @@ def main():
         varlist['dbx'] = efivar.EfiVar(ucs16.from_string('dbx'),
                                        authdata = dbxupdate)
 
+    if options.add_dbx:
+        if not varlist.get('dbx'):
+            varlist.add_dummy_dbx(guids.OvmfEnrollDefaultKeys)
+        for item in options.add_dbx:
+            logging.info('adding dbx update from %s', item)
+            with open(item, 'rb') as f:
+                dbxupdate = f.read()
+            update = efivar.EfiVar(ucs16.from_string('dbx'),
+                                   authdata = dbxupdate)
+            varlist['dbx'].sigdb_merge(update)
+
+    if options.dbx_cert:
+        for item in options.dbx_cert:
+            varlist.add_cert('dbx', item[0], item[1])
+
+    if options.dbx_hash:
+        for item in options.dbx_hash:
+            varlist.add_hash('dbx', item[0], item[1])
+
     if options.mok:
         for item in options.mok:
             varlist.add_cert('MokList', item[0], item[1])
 
-    if options.db_hash:
-        for item in options.db_hash:
-            varlist.add_hash('db', item[0], item[1])
-
     if options.mok_hash:
         for item in options.mok_hash:
             varlist.add_hash('MokList', item[0], item[1])
@@ -291,7 +340,7 @@ def main():
 
     if options.print:
         if options.verbose:
-            varlist.print_normal(options.hexdump)
+            varlist.print_normal(options.hexdump, options.hashes)
         else:
             varlist.print_compact()
 
@@ -314,6 +363,9 @@ def main():
         else:
             jstore.JsonVarStore.write_varstore(options.output_json, varlist)
 
+    if options.output_auth:
+        authfiles.AuthFilesVarStore.write_varstore(options.output_auth, varlist)
+
     return 0
 
 if __name__ == '__main__':
